;******************************************************************************
;
; V1INSTD.386 is a simple VxD that allows an MS-DOS application running in
; a DOS VM to determine whether a previous instance of the same application
; is running in another DOS VM.
;
; To load the VxD, add a DEVICE=V1INSTD.386 command to the [386Enh] section
; of SYSTEM.INI. To retrieve the V86-mode entry point, call interrupt 2Fh with
; AX=1684h and BX=7FE8h.
;
; Note: The device ID used by V1INSTD (7FE8h) is not registered. If you plan
; to write and distribute a similar VxD, be sure to obtain a valid and unique
; device ID from Microsoft.
;
;******************************************************************************

        .386p

;******************************************************************************
;                             I N C L U D E S
;******************************************************************************

        .XLIST
        INCLUDE VMM.INC
        .LIST

;******************************************************************************
;                             E Q U A T E S
;******************************************************************************

V1INSTD_Major_Ver               equ     1
V1INSTD_Minor_Ver               equ     0
V1INSTD_Device_ID               equ     7FE8h

SUCCESSFUL_COMPLETION           equ     0
ERROR_INVALID_ID                equ     1
ERROR_ALREADY_REGISTERED        equ     2
ERROR_INVALID_FUNCTION_CODE     equ     3

;******************************************************************************
;              V I R T U A L   D E V I C E   D E C L A R A T I O N
;******************************************************************************

Declare_Virtual_Device V1INSTD,\
                       V1INSTD_Major_Ver,\
                       V1INSTD_Minor_Ver,\
                       V1INSTD_Control,\
                       V1INSTD_Device_ID,\
                       Undefined_Init_Order,\
                       V1INSTD_V86_API_Proc,,

;******************************************************************************
;                                D A T A
;******************************************************************************

VxD_DATA_SEG

App_ID          dw      0               ;ID of registered app

VxD_DATA_ENDS

;******************************************************************************
;                      P A G E - L O C K E D   C O D E
;******************************************************************************

VxD_LOCKED_CODE_SEG

BeginProc V1INSTD_Control

        clc                             ;Clear the carry flag and return
        ret

EndProc V1INSTD_Control

VxD_LOCKED_CODE_ENDS

;******************************************************************************
;                               C O D E
;******************************************************************************

VxD_CODE_SEG

;------------------------------------------------------------------------------
; V1INSTD_V86_API_Proc is the exported V86-mode API procedure.
;
; V1INSTD supports two functions. The function code is passed in the
; caller's AX register. The functions are:
;
; AX=0: Register MS-DOS application. An MS-DOS app that wishes to prevent
;       subsequent instances of itself from being executed in other DOS VMs
;       must register itself with this API function when it is started. On
;       entry, BX holds a 16-bit ID code that identifies the application.
;       On return, AX holds one of the following return codes:
;
;         AX=0: The app was successfully registered
;         AX=1: Function failed because the app ID code is invalid
;         AX=2: Function failed because the app is already registered
;         AX=3: Invalid function code
;
;       Only one app can be registered at a time. This function will fail
;       if the app ID passed in BX is 0, another app is registered, or the
;       function code in AX is invalid.
;
; AX=1: Deregister MS-DOS application. An app that registered itself with
;       the previous function must call this function to deregister itself
;       before terminating. On entry, BX holds the application's ID code.
;       On return, AX holds one of the following return codes:
;
;         AX=0: The app was successfully deregistered
;         AX=1: Function failed because the app ID code is invalid
;         AX=3: Invalid function code
;
;       This function will fail if the ID code passed in BX is 0 or does
;       not match the ID code the app was registered with, or if the function
;       code in AX is invalid.
;------------------------------------------------------------------------------

BeginProc V1INSTD_V86_API_Proc

        mov     ax,[ebp.Client_AX]      ;Get the caller's AX into our AX
        or      ax,ax                   ;Check the value passed in AX
        jnz     V86_API_Function_1      ;Branch if AX!=0

        mov     ax,[ebp.Client_BX]      ;Get caller's BX into our AX
        or      ax,ax                   ;Error if the application ID code
        jz      V86_API_Error_1         ;passed in BX is 0

        cmp     [App_ID],0              ;Error if an app is already
        jne     V86_API_Error_2         ;registered

        mov     [App_ID],ax             ;Store the ID code and exit

        mov     [ebp.Client_AX],SUCCESSFUL_COMPLETION
        ret

V86_API_Function_1:
        cmp     ax,1                    ;Check the value passed in AX
        jne     V86_API_Error_3         ;Error if AX!=1

        mov     ax,[ebp.Client_BX]      ;Get caller's BX into our AX
        or      ax,ax                   ;Error if the application ID code
        jz      V86_API_Error_1         ;passed in BX is 0

        cmp     [App_ID],ax             ;Error if the ID code isn't the one
        jne     V86_API_Error_1         ;that the app was registered with

        mov     [App_ID],0              ;Zero the ID code and exit

        mov     [ebp.Client_AX],SUCCESSFUL_COMPLETION
        ret

V86_API_Error_1:
        mov     [ebp.Client_AX],ERROR_INVALID_ID
        ret

V86_API_Error_2:
        mov     [ebp.Client_AX],ERROR_ALREADY_REGISTERED
        ret

V86_API_Error_3:
        mov     [ebp.Client_AX],ERROR_INVALID_FUNCTION_CODE
        ret

EndProc V1INSTD_V86_API_Proc

VxD_CODE_ENDS

        END

